////////////////////////////////////////////////////////////////////////////
// CLangConfig 饹Υץơ
//
////////////////////////////////////////////////////////////////////////////

#include "../TaEdit_std.h"

#include "LangConfig.h"
#include "TxtColorMgr.h"
#include "cpp/TxtColorMgr_cpp.h"
#include "java/TxtColorMgr_java.h"
#include "cobol/TxtColorMgr_cbl.h"
#include "js/TxtColorMgr_js.h"
#include "vbs/TxtColorMgr_vbs.h"
#include "cs/TxtColorMgr_cs.h"
#include "txt/TxtColorMgr_txt.h"
#include "html/TxtColorMgr_html.h"

using namespace NLangExt;

////////////////////////////////////////////////////////////////////////////
// /˴
////////////////////////////////////////////////////////////////////////////

CLangConfig::CLangConfig( VClsPtr< CConfigInfo > pConf )
{
	StoreExtTypeInfo( pConf->RefCPP_FileExt(), LANGTYPE_CPP );
	StoreExtTypeInfo( pConf->RefJAVA_FileExt(), LANGTYPE_JAVA );
	StoreExtTypeInfo( pConf->RefCOBOL_FileExt(), LANGTYPE_CBL );
	StoreExtTypeInfo( pConf->RefJS_FileExt(), LANGTYPE_JS );
	StoreExtTypeInfo( pConf->RefVBS_FileExt(), LANGTYPE_VBS );
	StoreExtTypeInfo( pConf->RefCS_FileExt(), LANGTYPE_CS );
	StoreExtTypeInfo( pConf->RefHTML_FileExt(), LANGTYPE_HTML );
}

CLangConfig::~CLangConfig()
{

}

////////////////////////////////////////////////////////////////////////////
// ᥽å
////////////////////////////////////////////////////////////////////////////

// ե̾бॳɤ
enumLANGTYPE CLangConfig::GetLangTypeID( const char* FileName ) const
{
	assert( NULL != this && NULL != FileName );
	string ext = GetExt( FileName );
	map < string, enumLANGTYPE >::const_iterator itr = mapExtTypeInfo.find( ext );

	// ĥҤĤʤϡ̾Υƥȥեȸʤ
	if ( mapExtTypeInfo.end() == itr )
		return LANGTYPE_TEXT;
	return itr->second;
}

// ֥Ȥ
CTxtColorMgr* CLangConfig::CreateColorMgr( enumLANGTYPE type ) const
{
	assert( NULL != this );
	switch ( type ) {
	case LANGTYPE_CPP:	// C/C++
		return new NLangExt_cpp::CTxtColorMgr_cpp();
	case LANGTYPE_JAVA:	// Java
		return new NLangExt_java::CTxtColorMgr_java();
	case LANGTYPE_CBL:	// Cobol
		return new NLangExt_cbl::CTxtColorMgr_cbl();
	case LANGTYPE_JS:	// JavaScript
		return new NLangExt_js::CTxtColorMgr_js();
	case LANGTYPE_VBS:	// VBScript
		return new NLangExt_vbs::CTxtColorMgr_vbs();
	case LANGTYPE_CS:	// C#
		return new NLangExt_cs::CTxtColorMgr_cs();
	case LANGTYPE_HTML:	// HTML
		return new NLangExt_html::CTxtColorMgr_html();
	default:
		return new NLangExt_txt::CTxtColorMgr_txt();
	}
}

// 
const CLangConfig& CLangConfig::operator =( const CLangConfig& r )
{
	mapExtTypeInfo = r.mapExtTypeInfo;
	return (*this);
}

// ե̾γĥҤ
string CLangConfig::GetExt( const char* fname ) const
{
	assert( NULL != this );
	int i = strlen( fname );
	if ( i <= 0 ) return "";
	for ( --i; i >= 0 && fname[i] != '.' && fname[i] != '/'; --i );
	if ( i >= 0 && fname[i] == '.' )
		return fname + i + 1;
	else
		return "";
}

// CConfigInfomapExtTypeInfo˳Ǽ
void CLangConfig::StoreExtTypeInfo( const string& ExtInfo, enumLANGTYPE n )
{
	int i = 0;
	int j;
	while ( i < ExtInfo.length() ) {
		for ( j = i; j < ExtInfo.length() && ExtInfo[j] != ','; j++ );
		mapExtTypeInfo.insert( pair< string, enumLANGTYPE >( ExtInfo.substr( i, j - i ), n ) );
		i = j + 1;
	}
}

// ̤̾
enumLANGTYPE NLangExt::GetLangTypeID_ByLangName( const char *pArgLangName )
{
	assert( NULL != pArgLangName );

	// ̾ȸ̤
	const char *pLangName[] = {
		"Text", "C", "Java", "Cobol", "JS", "VBS", "CS", "HTML"
	};
	enum enumLANGTYPE vLangType[] = {
		LANGTYPE_TEXT, LANGTYPE_CPP, LANGTYPE_JAVA, LANGTYPE_CBL,
		LANGTYPE_JS, LANGTYPE_VBS, LANGTYPE_CS, LANGTYPE_HTML
	};
	int cnt = sizeof( pLangName ) / sizeof( pLangName[0] );
	int i;

	// 
	for ( i = 0; i < cnt && strcasecmp( pArgLangName, pLangName[i] ); i++ );
	if ( i >= cnt )
		return LANGTYPE_TEXT;
	else
		return vLangType[i];
}

// ̤̾
const char* NLangExt::GetLangName_ByLangTypeID( enumLANGTYPE type )
{
	// ̾ȸ̤
	const char *pLangName[] = {
		"Text", "C", "Java", "Cobol", "JS", "VBS", "CS", "HTML"
	};
	enum enumLANGTYPE vLangType[] = {
		LANGTYPE_TEXT, LANGTYPE_CPP, LANGTYPE_JAVA, LANGTYPE_CBL,
		LANGTYPE_JS, LANGTYPE_VBS, LANGTYPE_CS, LANGTYPE_HTML
	};
	int cnt = sizeof( pLangName ) / sizeof( pLangName[0] );
	int i;

	// 
	for ( i = 0; i < cnt && type != vLangType[i]; i++ );
	if ( i >= cnt )
		return "";
	else
		return pLangName[i];
}
